// Beginning of Joreskog-inspired CFA in C++
// dt 11/2004 CodeWarrior 8.3 / OS X

#include <ctime> 		 // for timing routines
#include "CFA1.h"
#include "DataTable.h"

int main() {
	using namespace std;
	clock_t time1, time2;
	char InputFileName[255]; 	// this is the space for the InputFileName (255 chars max)
	int N;
	int i, j;
	
	int nvar;		// number of observed variables
	int npar = 0;	// number of free parameters in the model
	int nfac;		// number of factors
	double N2; 		// (to be) sample size divided by 2
	SymmetricMatrix S;		// the observed covariance (correlation) matrix
	SymmetricMatrix Hinv(npar);	// the inverse hessian
	
	cout << "Enter N: ";	// this next 55 lines reads the input files
	cin >> N;				// it would better be moved into a separate routine
	N2 = float(N)/2;		// to "clean up" this main program
	cin.get();		// this call "picks up" the return-character after N,
					// so othe next call to cin.getline() works
	cout << "Enter covariance matrix file name: ";	
	cin.getline(InputFileName, 255);	
	DataTable S_input(InputFileName);	// input uses the "DataTable" class 
	nvar = S_input.nVars;				// developed for the regression example
	for (int v = 0; v < S_input.nVars; v++) {	
		cout << S_input.VarNames[v] << "\t";
	}	
	cout << endl;
	S.ReSize(nvar);
	S << S_input.Data;
    cout << "Observed covariance matrix:" << endl;
    cout << setprecision(4) << S;

	cout << "Enter starting Lambda file name: ";	
	cin.getline(InputFileName, 255);	
	DataTable L_input(InputFileName);	
	nfac = L_input.nVars;
    cout << "Starting Lambda:" << endl;
    cout << setprecision(4) << L_input.Data;

	cout << "Enter starting Delta file name: ";	
	cin.getline(InputFileName, 255);	
	DataTable D_input(InputFileName);	
    cout << "Starting Delta:" << endl;
    cout << setprecision(4) << D_input.Data;

	cout << "Enter Lambda parameter numbers file name: ";	
	cin.getline(InputFileName, 255);	
	DataTable LN_input(InputFileName);	
    cout << "Lambda parameter numbers:" << endl;
    cout << setprecision(0) << LN_input.Data;
    npar = LN_input.Data.Maximum();

	cout << "Enter Delta parameter numbers file name: ";	
	cin.getline(InputFileName, 255);	
	DataTable DN_input(InputFileName);	
    cout << "Delta parameter numbers:" << endl;
    cout << setprecision(0) << DN_input.Data;
	if (DN_input.Data.Maximum() > npar) npar = DN_input.Data.Maximum();
    cout << npar << endl;
    cout << setprecision(4);
	
// this segment extracts the starting values for the parameters
// from L_input.Data() & D_input.Data(), and puts them in the vector p
// at locations that are specified in the integer arrays
// LN_input.Data() & DN_input.Data()
	ColumnVector p(npar);
	ColumnVector para(npar);
	for (i=1; i <= nvar; i++) {
		for (j=1; j <= nfac; j++) {
			if (int(LN_input.Data(i,j)) != 0) {
						p(int(LN_input.Data(i,j))) = L_input.Data(i,j);
			}
		}
		if (int(DN_input.Data(i,1)) != 0) 
					p(int(DN_input.Data(i,1))) = D_input.Data(i,1);
	}

	// "CFA" is an object of class CFA_LL
	CFA_LL CFA(S, p); // loglikehood "object"
// the following 15 lines are initialization for CFA public variables
// this would be better placed in an initialization routine for class CFA
	CFA.N2 = N2;
	CFA.nvar = nvar;
	CFA.npar = npar;
	CFA.nfac = nfac;
	CFA.Lambda.ReSize(nvar, nfac);
	CFA.Lambda = L_input.Data;
	CFA.Delta.ReSize(nvar);
	CFA.iL.resize(nvar+1);
	for (int k=0; k<nvar+1; k++)
		CFA.iL[k].resize(nfac+1);
	CFA.iD.resize(nvar+1);
	for (i=1; i <= nvar; i++) {
		for (j=1; j <= nfac; j++) {
			CFA.iL[i][j] = int(LN_input.Data(i,j));
		}
		CFA.iD[i] = int(DN_input.Data(i,1));
	}

	
	MLE_D_FI mle_d_fi(CFA,50,0.0000001);    // mle "object" 

	
	para = p;
	time1 = clock();
	mle_d_fi.Fit(para);                     // do the fit
	time2 = clock();
	cout << setprecision(4) << resetiosflags(ios::right);
	cout << "Time required: " << float(time2-time1)/CLOCKS_PER_SEC << " seconds" << endl;

	cout << setprecision(3) << endl;
	cout << "Estimated parameters:" << endl << para << endl;
	Hinv = CFA.FI().i();
	cout << "Standard Errors:" << setprecision(3) << endl;
	for (int i=1; i <= npar; i++) cout << sqrt(Hinv(i,i)) << endl;
	cout << endl;

	cout << "Estimated Lambda:" << endl << CFA.Lambda << endl;
	cout << "Estimated Delta:" << endl << CFA.Delta << endl;

	return 0;
}